@@ -24,6 +24,7 @@
  */
 #include "FilterParameters/AbstractParameter.h"
 #include <QDebug>
+#include <QGridLayout>
 #include <cstring>
 #include "Common.h"
 #include "FilterParameters/BoolParameter.h"
@@ -40,8 +41,19 @@
 #include "FilterParameters/PointParameter.h"
 #include "FilterParameters/SeparatorParameter.h"
 #include "FilterParameters/TextParameter.h"
+#include "Logger.h"
 
-AbstractParameter::AbstractParameter(QObject * parent, bool actualParameter) : QObject(parent), _actualParameter(actualParameter), _update(true) {}
+const QStringList AbstractParameter::NoValueParameters = {"link", "note", "separator"};
+
+AbstractParameter::AbstractParameter(QObject * parent, bool actualParameter) : QObject(parent), _actualParameter(actualParameter)
+{
+  _update = true;
+  _defaultVisibilityState = VisibleParameter;
+  _visibilityState = VisibleParameter;
+  _visibilityPropagation = PropagateNone;
+  _row = -1;
+  _grid = nullptr;
+}
 
 AbstractParameter::~AbstractParameter() {}
 
@@ -74,7 +86,7 @@ void AbstractParameter::addToKeypointList(KeypointList &) const {}
 
 void AbstractParameter::extractPositionFromKeypointList(KeypointList &) {}
 
-AbstractParameter * AbstractParameter::createFromText(const char * text, int & length, QString & error, QObject * parent)
+AbstractParameter * AbstractParameter::createFromText(const char * text, int & length, QString & error, QWidget * parent)
 {
   AbstractParameter * result = 0;
   QString line = text;
@@ -140,11 +152,63 @@ AbstractParameter * AbstractParameter::createFromText(const char * text, int & l
   return result;
 }
 
+AbstractParameter::VisibilityState AbstractParameter::defaultVisibilityState() const
+{
+  return _defaultVisibilityState;
+}
+
+void AbstractParameter::setVisibilityState(AbstractParameter::VisibilityState state)
+{
+  if (!_grid || _row == -1) {
+    return;
+  }
+  if (state == UnspecifiedVisibilityState) {
+    setVisibilityState(defaultVisibilityState());
+    return;
+  }
+  _visibilityState = state;
+  for (int col = 0; col < 5; ++col) {
+    QLayoutItem * item = _grid->itemAtPosition(_row, col);
+    if (item) {
+      auto widget = item->widget();
+      switch (state & 3) {
+      case VisibleParameter:
+        widget->setEnabled(true);
+        widget->show();
+        break;
+      case DisabledParameter:
+        widget->setEnabled(false);
+        widget->show();
+        break;
+      case HiddenParameter:
+        widget->hide();
+        break;
+      case UnspecifiedVisibilityState:
+        // Taken care above (if)
+        break;
+      }
+    }
+  }
+}
+
+AbstractParameter::VisibilityState AbstractParameter::visibilityState() const
+{
+  return _visibilityState;
+}
+
+AbstractParameter::VisibilityPropagation AbstractParameter::visibilityPropagation() const
+{
+  return _visibilityPropagation;
+}
+
 QStringList AbstractParameter::parseText(const QString & type, const char * text, int & length)
 {
   QStringList result;
-  QString str = text;
+  const QString str = text;
   result << str.left(str.indexOf("=")).trimmed();
+#ifdef _GMIC_QT_DEBUG_
+  _debugName = result.back();
+#endif
 
   QRegExp re(QString("^[^=]*\\s*=\\s*(_?)%1\\s*(.)").arg(type), Qt::CaseInsensitive);
   re.indexIn(str);
@@ -165,6 +229,33 @@ QStringList AbstractParameter::parseText(const QString & type, const char * text
   }
   QString values = str.mid(prefixLength, -1).left(end - (text + prefixLength)).trimmed();
   length = 1 + end - text;
+
+  if (text[length] == '_' && text[length + 1] >= '0' && text[length + 1] <= '2') {
+    _defaultVisibilityState = static_cast<VisibilityState>(text[length + 1] - '0');
+    _visibilityPropagation = PropagateNone;
+    switch (text[length + 2]) {
+    case '-':
+      _visibilityPropagation = PropagateUp;
+      length += 3;
+      break;
+    case '+':
+      _visibilityPropagation = PropagateDown;
+      length += 3;
+      break;
+    case '*':
+      _visibilityPropagation = PropagateUpDown;
+      length += 3;
+      break;
+    default:
+      length += 2;
+      break;
+    }
+    if (NoValueParameters.contains(type)) {
+      Logger::log(QString("[gmic-qt] Warning: %1 parameter should not define visibility. Ignored.").arg(result.first()));
+      _defaultVisibilityState = AbstractParameter::VisibleParameter;
+      _visibilityPropagation = PropagateNone;
+    }
+  }
   while (text[length] && (text[length] == ',' || str[length].isSpace())) {
     ++length;
   }
